@@ -1,6 +1,6 @@
-! WHIZARD 2.2.3 Nov 30 2014
+! WHIZARD 2.2.5 Feb 27 2015
 ! 
-! Copyright (C) 1999-2014 by 
+! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -9,7 +9,8 @@
 !     Fabian Bach <fabian.bach@desy.de>
 !     Christian Speckner <cnspeckn@googlemail.com> 
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Felix Braam, Sebastian Schmidt, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam, 
+!     Sebastian Schmidt, Daniel Wiesler 
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -42,90 +43,83 @@ module flavors
   use physics_defs, only: HADRON_REMNANT_TRIPLET
   use physics_defs, only: HADRON_REMNANT_OCTET
   use model_data
-  use colors, only: color_t, color_init
+  use colors, only: color_t
 
   implicit none
   private
 
   public :: flavor_t
-  public :: flavor_init
-  public :: flavor_undefine
-  public :: flavor_write
-  public :: flavor_write_raw
-  public :: flavor_read_raw
-  public :: flavor_set_model
-  public :: flavor_is_defined
-  public :: flavor_is_valid
-  public :: flavor_is_associated
-  public :: flavor_get_pdg
-  public :: flavor_get_pdg_anti
-  public :: flavor_get_pdg_abs
-  public :: flavor_is_visible
-  public :: flavor_is_parton
-  public :: flavor_is_beam_remnant
-  public :: flavor_is_gauge
-  public :: flavor_is_left_handed
-  public :: flavor_is_right_handed
-  public :: flavor_is_antiparticle
-  public :: flavor_has_antiparticle
-  public :: flavor_is_stable
-  public :: flavor_get_decays
-  public :: flavor_decays_isotropically
-  public :: flavor_decays_diagonal
-  public :: flavor_is_polarized
-  public :: flavor_get_name
-  public :: flavor_get_tex_name
-  public :: flavor_get_spin_type
-  public :: flavor_get_multiplicity
-  public :: flavor_get_isospin_type
-  public :: flavor_get_charge_type
-  public :: flavor_get_color_type
-  public :: flavor_get_charge
-  public :: flavor_get_mass
-  public :: flavor_get_width
-  public :: flavor_get_isospin
-  public :: operator(.match.)
-  public :: operator(==)
-  public :: operator(/=)
   public :: operator(.merge.)
   public :: color_from_flavor
-  public :: flavor_anti
 
   type :: flavor_t
      private
      integer :: f = UNDEFINED
+     logical :: radiated = .false.
      type(field_data_t), pointer :: field_data => null ()
+   contains
+     generic :: init => &
+          flavor_init0_empty, &
+          flavor_init0, &
+          flavor_init0_field_data, &
+          flavor_init0_model, &
+          flavor_init0_model_alt, &
+          flavor_init0_name_model
+     procedure, private :: flavor_init0_empty
+     procedure, private :: flavor_init0
+     procedure, private :: flavor_init0_field_data
+     procedure, private :: flavor_init0_model
+     procedure, private :: flavor_init0_model_alt
+     procedure, private :: flavor_init0_name_model
+     procedure :: tag_radiated => flavor_tag_radiated
+     procedure :: undefine => flavor_undefine
+     procedure :: write => flavor_write
+     procedure :: write_raw => flavor_write_raw
+     procedure :: read_raw => flavor_read_raw
+     procedure :: set_model => flavor_set_model_single
+     procedure :: is_defined => flavor_is_defined
+     procedure :: is_valid => flavor_is_valid
+     procedure :: is_associated => flavor_is_associated
+     procedure :: is_radiated => flavor_is_radiated
+     procedure :: get_pdg => flavor_get_pdg
+     procedure :: get_pdg_anti => flavor_get_pdg_anti
+     procedure :: get_pdg_abs => flavor_get_pdg_abs
+     procedure :: is_visible => flavor_is_visible
+     procedure :: is_parton => flavor_is_parton
+     procedure :: is_beam_remnant => flavor_is_beam_remnant
+     procedure :: is_gauge => flavor_is_gauge
+     procedure :: is_left_handed => flavor_is_left_handed
+     procedure :: is_right_handed => flavor_is_right_handed
+     procedure :: is_antiparticle => flavor_is_antiparticle
+     procedure :: has_antiparticle => flavor_has_antiparticle
+     procedure :: is_stable => flavor_is_stable
+     procedure :: get_decays => flavor_get_decays
+     procedure :: decays_isotropically => flavor_decays_isotropically
+     procedure :: decays_diagonal => flavor_decays_diagonal
+     procedure :: has_decay_helicity => flavor_has_decay_helicity
+     procedure :: get_decay_helicity => flavor_get_decay_helicity
+     procedure :: is_polarized => flavor_is_polarized
+     procedure :: get_name => flavor_get_name
+     procedure :: get_tex_name => flavor_get_tex_name
+     procedure :: get_spin_type => flavor_get_spin_type
+     procedure :: get_multiplicity => flavor_get_multiplicity
+     procedure :: get_isospin_type => flavor_get_isospin_type
+     procedure :: get_charge_type => flavor_get_charge_type
+     procedure :: get_color_type => flavor_get_color_type
+     procedure :: get_charge => flavor_get_charge
+     procedure :: get_mass => flavor_get_mass
+     procedure :: get_width => flavor_get_width
+     procedure :: get_isospin => flavor_get_isospin
+     generic :: operator(.match.) => flavor_match
+     generic :: operator(==) => flavor_eq
+     generic :: operator(/=) => flavor_neq
+     procedure, private :: flavor_match
+     procedure, private :: flavor_eq
+     procedure, private :: flavor_neq
+     procedure :: anti => flavor_anti
   end type flavor_t
 
 
-  interface flavor_init
-     module procedure flavor_init0_empty
-     module procedure flavor_init0
-     module procedure flavor_init0_field_data
-     module procedure flavor_init0_model
-     module procedure flavor_init0_model_alt
-     module procedure flavor_init0_name_model
-     module procedure flavor_init1_model
-     module procedure flavor_init1_model_alt
-     module procedure flavor_init1_name_model
-     module procedure flavor_init2_model
-     module procedure flavor_init2_model_alt
-!     module procedure flavor_init_aval_model
-  end interface
-  interface flavor_set_model
-     module procedure flavor_set_model_single
-     module procedure flavor_set_model_array
-     module procedure flavor_set_model_array2
-  end interface
-  interface operator(.match.)
-     module procedure flavor_match
-  end interface
-  interface operator(==)
-     module procedure flavor_eq
-  end interface
-  interface operator(/=)
-     module procedure flavor_neq
-  end interface
   interface operator(.merge.)
      module procedure merge_flavors0
      module procedure merge_flavors1
@@ -139,35 +133,43 @@ module flavors
 contains
 
   elemental subroutine flavor_init0_empty (flv)
-    type(flavor_t), intent(out) :: flv
+    class(flavor_t), intent(inout) :: flv
+    flv%f = UNDEFINED
+    flv%radiated = .false.
+    flv%field_data => null ()
   end subroutine flavor_init0_empty
 
   elemental subroutine flavor_init0 (flv, f)
-    type(flavor_t), intent(out) :: flv
+    class(flavor_t), intent(inout) :: flv
     integer, intent(in) :: f
     flv%f = f
+    flv%radiated = .false.
+    flv%field_data => null ()
   end subroutine flavor_init0
 
-  subroutine flavor_init0_field_data (flv, field_data)
-    type(flavor_t), intent(out) :: flv
+  impure elemental subroutine flavor_init0_field_data (flv, field_data)
+    class(flavor_t), intent(inout) :: flv
     type(field_data_t), intent(in), target :: field_data
     flv%f = field_data%get_pdg ()
+    flv%radiated = .false.
     flv%field_data => field_data
   end subroutine flavor_init0_field_data
 
-  subroutine flavor_init0_model (flv, f, model)
-    type(flavor_t), intent(out) :: flv
+  impure elemental subroutine flavor_init0_model (flv, f, model)
+    class(flavor_t), intent(inout) :: flv
     integer, intent(in) :: f
     class(model_data_t), intent(in), target :: model
     flv%f = f
+    flv%radiated = .false.
     flv%field_data => model%get_field_ptr (f, check=.true.)
   end subroutine flavor_init0_model
 
-  subroutine flavor_init0_model_alt (flv, f, model, alt_model)
-    type(flavor_t), intent(out) :: flv
+  impure elemental subroutine flavor_init0_model_alt (flv, f, model, alt_model)
+    class(flavor_t), intent(inout) :: flv
     integer, intent(in) :: f
     class(model_data_t), intent(in), target :: model, alt_model
     flv%f = f
+    flv%radiated = .false.
     flv%field_data => model%get_field_ptr (f, check=.false.)
     if (.not. associated (flv%field_data)) then
        flv%field_data => alt_model%get_field_ptr (f, check=.false.)
@@ -181,82 +183,78 @@ contains
     end if
   end subroutine flavor_init0_model_alt
 
-  subroutine flavor_init1_model (flv, f, model)
-    type(flavor_t), dimension(:), intent(out) :: flv
-    integer, dimension(:), intent(in) :: f
-    class(model_data_t), intent(in), target :: model
-    integer :: i
-    do i = 1, size (f)
-       call flavor_init0_model (flv(i), f(i), model)
-    end do
-  end subroutine flavor_init1_model
-
-  subroutine flavor_init1_model_alt (flv, f, model, alt_model)
-    type(flavor_t), dimension(:), intent(out) :: flv
-    integer, dimension(:), intent(in) :: f
-    class(model_data_t), intent(in), target :: model, alt_model
-    integer :: i
-    do i = 1, size (f)
-       call flavor_init0_model_alt (flv(i), f(i), model, alt_model)
-    end do
-  end subroutine flavor_init1_model_alt
-
-  subroutine flavor_init2_model (flv, f, model)
-    type(flavor_t), dimension(:,:), intent(out) :: flv
-    integer, dimension(:,:), intent(in) :: f
-    class(model_data_t), intent(in), target :: model
-    integer :: i
-    do i = 1, size (f, 2)
-       call flavor_init1_model (flv(:,i), f(:,i), model)
-    end do
-  end subroutine flavor_init2_model
-
-  subroutine flavor_init2_model_alt (flv, f, model, alt_model)
-    type(flavor_t), dimension(:,:), intent(out) :: flv
-    integer, dimension(:,:), intent(in) :: f
-    class(model_data_t), intent(in), target :: model, alt_model
-    integer :: i
-    do i = 1, size (f, 2)
-       call flavor_init1_model_alt (flv(:,i), f(:,i), model, alt_model)
-    end do
-  end subroutine flavor_init2_model_alt
-
-  subroutine flavor_init0_name_model (flv, name, model)
-    type(flavor_t), intent(out) :: flv
+!   subroutine flavor_init1_model (flv, f, model)
+!     type(flavor_t), dimension(:), intent(out) :: flv
+!     integer, dimension(:), intent(in) :: f
+!     class(model_data_t), intent(in), target :: model
+!     integer :: i
+!     do i = 1, size (f)
+!        call flavor_init0_model (flv(i), f(i), model)
+!     end do
+!   end subroutine flavor_init1_model
+! 
+!   subroutine flavor_init1_model_alt (flv, f, model, alt_model)
+!     type(flavor_t), dimension(:), intent(out) :: flv
+!     integer, dimension(:), intent(in) :: f
+!     class(model_data_t), intent(in), target :: model, alt_model
+!     integer :: i
+!     do i = 1, size (f)
+!        call flavor_init0_model_alt (flv(i), f(i), model, alt_model)
+!     end do
+!   end subroutine flavor_init1_model_alt
+! 
+!   subroutine flavor_init2_model (flv, f, model)
+!     type(flavor_t), dimension(:,:), intent(out) :: flv
+!     integer, dimension(:,:), intent(in) :: f
+!     class(model_data_t), intent(in), target :: model
+!     integer :: i
+!     do i = 1, size (f, 2)
+!        call flavor_init1_model (flv(:,i), f(:,i), model)
+!     end do
+!   end subroutine flavor_init2_model
+! 
+!   subroutine flavor_init2_model_alt (flv, f, model, alt_model)
+!     type(flavor_t), dimension(:,:), intent(out) :: flv
+!     integer, dimension(:,:), intent(in) :: f
+!     class(model_data_t), intent(in), target :: model, alt_model
+!     integer :: i
+!     do i = 1, size (f, 2)
+!        call flavor_init1_model_alt (flv(:,i), f(:,i), model, alt_model)
+!     end do
+!   end subroutine flavor_init2_model_alt
+! 
+  impure elemental subroutine flavor_init0_name_model (flv, name, model)
+    class(flavor_t), intent(inout) :: flv
     type(string_t), intent(in) :: name
     class(model_data_t), intent(in), target :: model
-    flv%field_data => model%get_field_ptr (name, check=.true.)
     flv%f = model%get_pdg (name)
+    flv%radiated = .false.
+    flv%field_data => model%get_field_ptr (name, check=.true.)
   end subroutine flavor_init0_name_model
-
-  subroutine flavor_init1_name_model (flv, name, model)
-    type(flavor_t), dimension(:), intent(out) :: flv
-    type(string_t), dimension(:), intent(in) :: name
-    class(model_data_t), intent(in), target :: model
-    integer :: i
-    do i = 1, size (name)
-       call flavor_init0_name_model (flv(i), name(i), model)
-    end do
-  end subroutine flavor_init1_name_model
-
-!   subroutine flavor_init_aval_model (flv, aval, model)
-!     type(flavor_t), dimension(:), intent(out), allocatable :: flv
-!     type(pdg_array_t), intent(in) :: aval
+! 
+!   subroutine flavor_init1_name_model (flv, name, model)
+!     type(flavor_t), dimension(:), intent(out) :: flv
+!     type(string_t), dimension(:), intent(in) :: name
 !     class(model_data_t), intent(in), target :: model
-!     integer, dimension(:), allocatable :: pdg
-!     pdg = aval
-!     allocate (flv (size (pdg)))
-!     call flavor_init (flv, pdg, model)
-!   end subroutine flavor_init_aval_model
+!     integer :: i
+!     do i = 1, size (name)
+!        call flavor_init0_name_model (flv(i), name(i), model)
+!     end do
+!   end subroutine flavor_init1_name_model
 
+  elemental subroutine flavor_tag_radiated (flv)
+    class(flavor_t), intent(inout) :: flv
+    flv%radiated = .true.
+  end subroutine flavor_tag_radiated
+  
   elemental subroutine flavor_undefine (flv)
-    type(flavor_t), intent(inout) :: flv
+    class(flavor_t), intent(inout) :: flv
     flv%f = UNDEFINED
     flv%field_data => null ()
   end subroutine flavor_undefine
 
   subroutine flavor_write (flv, unit)
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     integer, intent(in), optional :: unit
     integer :: u
     u = given_output_unit (unit);  if (u < 0)  return
@@ -266,78 +264,70 @@ contains
        write (u, "(A)", advance="no")  "p("
     end if
     write (u, "(I0)", advance="no")  flv%f
+    if (flv%radiated) then
+       write (u, "('*')", advance="no")
+    end if
     write (u, "(A)", advance="no")  ")"
   end subroutine flavor_write
 
   subroutine flavor_write_raw (flv, u)
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     integer, intent(in) :: u
     write (u) flv%f
+    write (u) flv%radiated
   end subroutine flavor_write_raw
 
   subroutine flavor_read_raw (flv, u, iostat)
-    type(flavor_t), intent(out) :: flv
+    class(flavor_t), intent(out) :: flv
     integer, intent(in) :: u
     integer, intent(out), optional :: iostat
     read (u, iostat=iostat) flv%f
+    if (present (iostat)) then
+       if (iostat /= 0)  return
+    end if
+    read (u, iostat=iostat)  flv%radiated
   end subroutine flavor_read_raw
 
-  subroutine flavor_set_model_single (flv, model)
-    type(flavor_t), intent(inout) :: flv
+  impure elemental subroutine flavor_set_model_single (flv, model)
+    class(flavor_t), intent(inout) :: flv
     class(model_data_t), intent(in), target :: model
     if (flv%f /= UNDEFINED) &
          flv%field_data => model%get_field_ptr (flv%f)
   end subroutine flavor_set_model_single
 
-  subroutine flavor_set_model_array (flv, model)
-    type(flavor_t), dimension(:), intent(inout) :: flv
-    class(model_data_t), intent(in), target :: model
-    integer :: i
-    do i = 1, size (flv)
-       if (flv(i)%f /= UNDEFINED) &
-            flv(i)%field_data => model%get_field_ptr (flv(i)%f)
-    end do
-  end subroutine flavor_set_model_array
-
-  subroutine flavor_set_model_array2 (flv, model)
-    type(flavor_t), dimension(:,:), intent(inout) :: flv
-    class(model_data_t), intent(in), target :: model
-    integer :: i, j
-    do i = 1, size (flv, 2)
-       do j = 1, size (flv, 1)
-          if (flv(j,i)%f /= UNDEFINED) &
-               flv(j,i)%field_data => model%get_field_ptr (flv(j,i)%f)
-       end do
-    end do
-  end subroutine flavor_set_model_array2
-
   elemental function flavor_is_defined (flv) result (defined)
+    class(flavor_t), intent(in) :: flv
     logical :: defined
-    type(flavor_t), intent(in) :: flv
     defined = flv%f /= UNDEFINED
   end function flavor_is_defined
 
   elemental function flavor_is_valid (flv) result (valid)
+    class(flavor_t), intent(in) :: flv
     logical :: valid
-    type(flavor_t), intent(in) :: flv
     valid = flv%f /= INVALID
   end function flavor_is_valid
 
   elemental function flavor_is_associated (flv) result (flag)
+    class(flavor_t), intent(in) :: flv
     logical :: flag
-    type(flavor_t), intent(in) :: flv
     flag = associated (flv%field_data)
   end function flavor_is_associated
 
+  elemental function flavor_is_radiated (flv) result (flag)
+    class(flavor_t), intent(in) :: flv
+    logical :: flag
+    flag = flv%radiated
+  end function flavor_is_radiated
+
   elemental function flavor_get_pdg (flv) result (f)
     integer :: f
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     f = flv%f
   end function flavor_get_pdg
 
   elemental function flavor_get_pdg_anti (flv) result (f)
     integer :: f
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        if (flv%field_data%has_antiparticle ()) then
           f = -flv%f
@@ -351,13 +341,13 @@ contains
 
   elemental function flavor_get_pdg_abs (flv) result (f)
     integer :: f
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     f = abs (flv%f)
   end function flavor_get_pdg_abs
 
   elemental function flavor_is_visible (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        flag = flv%field_data%is_visible ()
     else
@@ -367,7 +357,7 @@ contains
 
   elemental function flavor_is_parton (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        flag = flv%field_data%is_parton ()
     else
@@ -377,7 +367,7 @@ contains
 
   elemental function flavor_is_beam_remnant (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     select case (abs (flv%f))
     case (HADRON_REMNANT, &
          HADRON_REMNANT_SINGLET, HADRON_REMNANT_TRIPLET, HADRON_REMNANT_OCTET)
@@ -389,7 +379,7 @@ contains
 
   elemental function flavor_is_gauge (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        flag = flv%field_data%is_gauge ()
     else
@@ -399,7 +389,7 @@ contains
 
   elemental function flavor_is_left_handed (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        if (flv%f > 0) then
           flag = flv%field_data%is_left_handed ()
@@ -413,7 +403,7 @@ contains
 
   elemental function flavor_is_right_handed (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        if (flv%f > 0) then
           flag = flv%field_data%is_right_handed ()
@@ -427,13 +417,13 @@ contains
 
   elemental function flavor_is_antiparticle (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     flag = flv%f < 0
   end function flavor_is_antiparticle
 
   elemental function flavor_has_antiparticle (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        flag = flv%field_data%has_antiparticle ()
     else
@@ -443,7 +433,7 @@ contains
 
   elemental function flavor_is_stable (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        flag = flv%field_data%is_stable (anti = flv%f < 0)
     else
@@ -452,7 +442,7 @@ contains
   end function flavor_is_stable
 
   subroutine flavor_get_decays (flv, decay)
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     type(string_t), dimension(:), intent(out), allocatable :: decay
     logical :: anti
     anti = flv%f < 0
@@ -463,7 +453,7 @@ contains
 
   elemental function flavor_decays_isotropically (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        flag = flv%field_data%decays_isotropically (anti = flv%f < 0)
     else
@@ -473,7 +463,7 @@ contains
 
   elemental function flavor_decays_diagonal (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        flag = flv%field_data%decays_diagonal (anti = flv%f < 0)
     else
@@ -481,9 +471,29 @@ contains
     end if
   end function flavor_decays_diagonal
 
+  elemental function flavor_has_decay_helicity (flv) result (flag)
+    logical :: flag
+    class(flavor_t), intent(in) :: flv
+    if (associated (flv%field_data)) then
+       flag = flv%field_data%has_decay_helicity (anti = flv%f < 0)
+    else
+       flag = .false.
+    end if
+  end function flavor_has_decay_helicity
+
+  elemental function flavor_get_decay_helicity (flv) result (hel)
+    integer :: hel
+    class(flavor_t), intent(in) :: flv
+    if (associated (flv%field_data)) then
+       hel = flv%field_data%decay_helicity (anti = flv%f < 0)
+    else
+       hel = 0
+    end if
+  end function flavor_get_decay_helicity
+
   elemental function flavor_is_polarized (flv) result (flag)
     logical :: flag
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        flag = flv%field_data%is_polarized (anti = flv%f < 0)
     else
@@ -493,7 +503,7 @@ contains
 
   elemental function flavor_get_name (flv) result (name)
     type(string_t) :: name
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        name = flv%field_data%get_name (flv%f < 0)
     else
@@ -503,7 +513,7 @@ contains
        
   elemental function flavor_get_tex_name (flv) result (name)
     type(string_t) :: name
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        name = flv%field_data%get_tex_name (flv%f < 0)
     else
@@ -513,7 +523,7 @@ contains
        
   elemental function flavor_get_spin_type (flv) result (type)
     integer :: type
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        type = flv%field_data%get_spin_type ()
     else
@@ -523,7 +533,7 @@ contains
 
   elemental function flavor_get_multiplicity (flv) result (type)
     integer :: type
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        type = flv%field_data%get_multiplicity ()
     else
@@ -533,7 +543,7 @@ contains
 
   elemental function flavor_get_isospin_type (flv) result (type)
     integer :: type
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        type = flv%field_data%get_isospin_type ()
     else
@@ -543,7 +553,7 @@ contains
 
   elemental function flavor_get_charge_type (flv) result (type)
     integer :: type
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        type = flv%field_data%get_charge_type ()
     else
@@ -553,13 +563,16 @@ contains
 
   elemental function flavor_get_color_type (flv) result (type)
     integer :: type
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        if (flavor_is_antiparticle (flv)) then
           type = - flv%field_data%get_color_type ()
        else
           type = flv%field_data%get_color_type ()
        end if
+       select case (type)
+       case (-1,-8);  type = abs (type)
+       end select
     else
        type = 1
     end if
@@ -567,7 +580,7 @@ contains
 
   elemental function flavor_get_charge (flv) result (charge)
     real(default) :: charge
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        if (flavor_is_antiparticle (flv)) then
           charge = flv%field_data%get_charge ()
@@ -581,7 +594,7 @@ contains
 
   elemental function flavor_get_mass (flv) result (mass)
     real(default) :: mass
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        mass = flv%field_data%get_mass ()
     else
@@ -591,7 +604,7 @@ contains
 
   elemental function flavor_get_width (flv) result (width)
     real(default) :: width
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        width = flv%field_data%get_width ()
     else
@@ -601,7 +614,7 @@ contains
 
   elemental function flavor_get_isospin (flv) result (isospin)
     real(default) :: isospin
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (associated (flv%field_data)) then
        if (flavor_is_antiparticle (flv)) then
           isospin = flv%field_data%get_isospin ()
@@ -615,7 +628,7 @@ contains
 
   elemental function flavor_match (flv1, flv2) result (eq)
     logical :: eq
-    type(flavor_t), intent(in) :: flv1, flv2
+    class(flavor_t), intent(in) :: flv1, flv2
     if (flv1%f /= UNDEFINED .and. flv2%f /= UNDEFINED) then
        eq = flv1%f == flv2%f
     else
@@ -625,7 +638,7 @@ contains
 
   elemental function flavor_eq (flv1, flv2) result (eq)
     logical :: eq
-    type(flavor_t), intent(in) :: flv1, flv2
+    class(flavor_t), intent(in) :: flv1, flv2
     if (flv1%f /= UNDEFINED .and. flv2%f /= UNDEFINED) then
        eq = flv1%f == flv2%f
     else if (flv1%f == UNDEFINED .and. flv2%f == UNDEFINED) then
@@ -637,7 +650,7 @@ contains
 
   elemental function flavor_neq (flv1, flv2) result (neq)
     logical :: neq
-    type(flavor_t), intent(in) :: flv1, flv2
+    class(flavor_t), intent(in) :: flv1, flv2
     if (flv1%f /= UNDEFINED .and. flv2%f /= UNDEFINED) then
        neq = flv1%f /= flv2%f
     else if (flv1%f == UNDEFINED .and. flv2%f == UNDEFINED) then
@@ -683,15 +696,16 @@ contains
     rev = .false.;  if (present (reverse)) rev = reverse
     select case (flavor_get_color_type (flv))
     case (1)
+       call col%init ()
     case (3)
-       call color_init (col, [c]);  c = c + 1
+       call col%init ([c]);  c = c + 1
     case (-3)
-       call color_init (col, [-c]);  c = c + 1
+       call col%init ([-c]);  c = c + 1
     case (8)
        if (rev) then
-          call color_init (col, [c+1, -c]);  c = c + 2
+          call col%init ([c+1, -c]);  c = c + 2
        else
-          call color_init (col, [c, -(c+1)]);  c = c + 2
+          call col%init ([c, -(c+1)]);  c = c + 2
        end if
     end select
   end function color_from_flavor0
@@ -710,7 +724,7 @@ contains
 
   function flavor_anti (flv) result (aflv)
     type(flavor_t) :: aflv
-    type(flavor_t), intent(in) :: flv
+    class(flavor_t), intent(in) :: flv
     if (flavor_has_antiparticle (flv)) then
        aflv%f = - flv%f
     else
