@@ -1,6 +1,6 @@
-! WHIZARD 2.2.3 Nov 30 2014
+! WHIZARD 2.2.5 Feb 27 2015
 ! 
-! Copyright (C) 1999-2014 by 
+! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -9,7 +9,8 @@
 !     Fabian Bach <fabian.bach@desy.de>
 !     Christian Speckner <cnspeckn@googlemail.com> 
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Felix Braam, Sebastian Schmidt, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam, 
+!     Sebastian Schmidt, Daniel Wiesler 
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -41,63 +42,77 @@ module quantum_numbers
   private
 
   public :: quantum_numbers_t
-  public :: quantum_numbers_init
   public :: quantum_numbers_write
-  public :: quantum_numbers_write_raw
-  public :: quantum_numbers_read_raw
   public :: quantum_numbers_get_flavor
   public :: quantum_numbers_get_color
   public :: quantum_numbers_get_helicity
-  public :: quantum_numbers_set_flavor
-  public :: quantum_numbers_set_color
-  public :: quantum_numbers_set_helicity
-  public :: quantum_numbers_set_color_ghost
-  public :: quantum_numbers_set_helicity_ghost
-  public :: quantum_numbers_set_model
   public :: quantum_numbers_get_color_type
-  public :: quantum_numbers_are_valid
-  public :: quantum_numbers_are_associated
-  public :: quantum_numbers_are_diagonal
-  public :: quantum_numbers_is_color_ghost
-  public :: quantum_numbers_is_helicity_ghost
-  public :: operator(.match.)
-  public :: operator(.fmatch.)
-  public :: operator(.fhmatch.)
-  public :: operator(.dhmatch.)
-  public :: operator(==)
-  public :: operator(/=)
   public :: quantum_numbers_are_compatible
   public :: quantum_numbers_are_physical
   public :: quantum_numbers_canonicalize_color
-  public :: quantum_numbers_set_color_map
+  public :: make_color_map
   public :: quantum_numbers_translate_color
   public :: quantum_numbers_get_max_color_value
-  public :: quantum_numbers_add_color_offset
   public :: quantum_number_array_make_color_contractions
-  public :: quantum_numbers_invert_color
   public :: operator(.merge.)
   public :: quantum_numbers_mask_t
-  public :: new_quantum_numbers_mask
-  public :: quantum_numbers_mask_init
+  public :: quantum_numbers_mask
   public :: quantum_numbers_mask_write
-  public :: quantum_numbers_mask_set_flavor
-  public :: quantum_numbers_mask_set_color
-  public :: quantum_numbers_mask_set_helicity
-  public :: quantum_numbers_mask_assign
   public :: any
-  public :: operator(.or.)
-  public :: operator(.eqv.)
-  public :: operator(.neqv.)
-  public :: quantum_numbers_undefine
   public :: quantum_numbers_undefined
-  public :: quantum_numbers_are_redundant
-  public :: quantum_numbers_mask_diagonal_helicity
 
   type :: quantum_numbers_t
      private
      type(flavor_t) :: f
      type(color_t) :: c
      type(helicity_t) :: h
+   contains
+     generic :: init => &
+        quantum_numbers_init0_f, &
+        quantum_numbers_init0_c, &
+        quantum_numbers_init0_h, &
+        quantum_numbers_init0_fc, &
+        quantum_numbers_init0_fh, &
+        quantum_numbers_init0_ch, &
+        quantum_numbers_init0_fch
+     procedure, private :: quantum_numbers_init0_f
+     procedure, private :: quantum_numbers_init0_c
+     procedure, private :: quantum_numbers_init0_h
+     procedure, private :: quantum_numbers_init0_fc
+     procedure, private :: quantum_numbers_init0_fh
+     procedure, private :: quantum_numbers_init0_ch
+     procedure, private :: quantum_numbers_init0_fch
+     procedure :: write => quantum_numbers_write_single
+     procedure :: write_raw => quantum_numbers_write_raw
+     procedure :: read_raw => quantum_numbers_read_raw
+     procedure :: get_flavor => quantum_numbers_get_flavor
+     procedure :: get_color => quantum_numbers_get_color
+     procedure :: get_helicity => quantum_numbers_get_helicity
+     procedure :: set_color_ghost => quantum_numbers_set_color_ghost
+     procedure :: set_model => quantum_numbers_set_model
+     procedure :: tag_radiated => quantum_numbers_tag_radiated
+     procedure :: get_color_type => quantum_numbers_get_color_type
+     procedure :: are_valid => quantum_numbers_are_valid
+     procedure :: are_associated => quantum_numbers_are_associated
+     procedure :: are_diagonal => quantum_numbers_are_diagonal
+     procedure :: is_color_ghost => quantum_numbers_is_color_ghost
+     generic :: operator(.match.) => quantum_numbers_match
+     generic :: operator(.fmatch.) => quantum_numbers_match_f
+     generic :: operator(.fhmatch.) => quantum_numbers_match_fh
+     generic :: operator(.dhmatch.) => quantum_numbers_match_hel_diag
+     generic :: operator(==) => quantum_numbers_eq
+     generic :: operator(/=) => quantum_numbers_neq
+     procedure, private :: quantum_numbers_match
+     procedure, private :: quantum_numbers_match_f
+     procedure, private :: quantum_numbers_match_fh
+     procedure, private :: quantum_numbers_match_hel_diag
+     procedure, private :: quantum_numbers_eq
+     procedure, private :: quantum_numbers_neq
+     procedure :: add_color_offset => quantum_numbers_add_color_offset
+     procedure :: invert_color => quantum_numbers_invert_color
+     procedure :: undefine => quantum_numbers_undefine
+     procedure :: undefined => quantum_numbers_undefined0
+     procedure :: are_redundant => quantum_numbers_are_redundant
   end type quantum_numbers_t
 
   type :: quantum_numbers_mask_t
@@ -107,63 +122,31 @@ module quantum_numbers
      logical :: cg = .false.
      logical :: h = .false.
      logical :: hd = .false.
+   contains
+     procedure :: init => quantum_numbers_mask_init
+     procedure :: write => quantum_numbers_mask_write_single
+     procedure :: set_flavor => quantum_numbers_mask_set_flavor
+     procedure :: set_color => quantum_numbers_mask_set_color
+     procedure :: set_helicity => quantum_numbers_mask_set_helicity
+     procedure :: assign => quantum_numbers_mask_assign
+     generic :: operator(.or.) => quantum_numbers_mask_or
+     procedure, private :: quantum_numbers_mask_or
+     generic :: operator(.eqv.) => quantum_numbers_mask_eqv
+     generic :: operator(.neqv.) => quantum_numbers_mask_neqv
+     procedure, private :: quantum_numbers_mask_eqv
+     procedure, private :: quantum_numbers_mask_neqv
+     procedure :: diagonal_helicity => quantum_numbers_mask_diagonal_helicity
   end type quantum_numbers_mask_t
 
 
-  interface quantum_numbers_init
-     module procedure quantum_numbers_init0_f
-     module procedure quantum_numbers_init0_c
-     module procedure quantum_numbers_init0_h
-     module procedure quantum_numbers_init0_fc
-     module procedure quantum_numbers_init0_fh
-     module procedure quantum_numbers_init0_ch
-     module procedure quantum_numbers_init0_fch
-     module procedure quantum_numbers_init1_f
-     module procedure quantum_numbers_init1_c
-     module procedure quantum_numbers_init1_h
-     module procedure quantum_numbers_init1_fc
-     module procedure quantum_numbers_init1_fh
-     module procedure quantum_numbers_init1_ch
-     module procedure quantum_numbers_init1_fch
-  end interface
-
   interface quantum_numbers_write
      module procedure quantum_numbers_write_single
      module procedure quantum_numbers_write_array
   end interface
-  interface quantum_numbers_get_flavor
-     module procedure quantum_numbers_get_flavor0
-     module procedure quantum_numbers_get_flavor1
-  end interface
-
-  interface quantum_numbers_set_flavor
-     module procedure quantum_numbers_set_flavor0
-     module procedure quantum_numbers_set_flavor1
-  end interface
-
-  interface quantum_numbers_set_model
-     module procedure quantum_numbers_set_model_single
-     module procedure quantum_numbers_set_model_array
-  end interface
-
-  interface operator(.match.)
-     module procedure quantum_numbers_match
-  end interface
-  interface operator(.fmatch.)
-     module procedure quantum_numbers_match_f
-  end interface
-  interface operator(.fhmatch.)
-     module procedure quantum_numbers_match_fh
-  end interface
-  interface operator(.dhmatch.)
-     module procedure quantum_numbers_match_hel_diag
-  end interface
-  interface operator(==)
-     module procedure quantum_numbers_eq
-  end interface
-  interface operator(/=)
-     module procedure quantum_numbers_neq
-  end interface
+  interface make_color_map
+     module procedure quantum_numbers_make_color_map
+  end interface make_color_map
+  
   interface quantum_numbers_translate_color
      module procedure quantum_numbers_translate_color0
      module procedure quantum_numbers_translate_color1
@@ -187,16 +170,6 @@ module quantum_numbers
   interface any
      module procedure quantum_numbers_mask_any
   end interface
-  interface operator(.or.)
-     module procedure quantum_numbers_mask_or
-  end interface
-
-  interface operator(.eqv.)
-     module procedure quantum_numbers_mask_eqv
-  end interface
-  interface operator(.neqv.)
-     module procedure quantum_numbers_mask_neqv
-  end interface
   interface quantum_numbers_undefined
      module procedure quantum_numbers_undefined0
      module procedure quantum_numbers_undefined1
@@ -206,50 +179,59 @@ module quantum_numbers
 
 contains
 
-  subroutine quantum_numbers_init0_f (qn, flv)
-    type(quantum_numbers_t), intent(out) :: qn
+  impure elemental subroutine quantum_numbers_init0_f (qn, flv)
+    class(quantum_numbers_t), intent(inout) :: qn
     type(flavor_t), intent(in) :: flv
     qn%f = flv
+    call qn%c%undefine ()
+    call qn%h%undefine ()
   end subroutine quantum_numbers_init0_f
 
-  subroutine quantum_numbers_init0_c (qn, col)
-    type(quantum_numbers_t), intent(out) :: qn
+  elemental subroutine quantum_numbers_init0_c (qn, col)
+    class(quantum_numbers_t), intent(inout) :: qn
     type(color_t), intent(in) :: col
+    call qn%f%undefine ()
     qn%c = col
+    call qn%h%undefine ()
   end subroutine quantum_numbers_init0_c
 
-  subroutine quantum_numbers_init0_h (qn, hel)
-    type(quantum_numbers_t), intent(out) :: qn
+  elemental subroutine quantum_numbers_init0_h (qn, hel)
+    class(quantum_numbers_t), intent(inout) :: qn
     type(helicity_t), intent(in) :: hel
+    call qn%f%undefine ()
+    call qn%c%undefine ()
     qn%h = hel
   end subroutine quantum_numbers_init0_h
 
-  subroutine quantum_numbers_init0_fc (qn, flv, col)
-    type(quantum_numbers_t), intent(out) :: qn
+  impure elemental subroutine quantum_numbers_init0_fc (qn, flv, col)
+    class(quantum_numbers_t), intent(inout) :: qn
     type(flavor_t), intent(in) :: flv
     type(color_t), intent(in) :: col
     qn%f = flv
     qn%c = col
+    call qn%h%undefine ()
   end subroutine quantum_numbers_init0_fc
 
-  subroutine quantum_numbers_init0_fh (qn, flv, hel)
-    type(quantum_numbers_t), intent(out) :: qn
+  impure elemental subroutine quantum_numbers_init0_fh (qn, flv, hel)
+    class(quantum_numbers_t), intent(inout) :: qn
     type(flavor_t), intent(in) :: flv
     type(helicity_t), intent(in) :: hel
     qn%f = flv
+    call qn%c%undefine ()
     qn%h = hel
   end subroutine quantum_numbers_init0_fh
 
-  subroutine quantum_numbers_init0_ch (qn, col, hel)
-    type(quantum_numbers_t), intent(out) :: qn
+  elemental subroutine quantum_numbers_init0_ch (qn, col, hel)
+    class(quantum_numbers_t), intent(inout) :: qn
     type(color_t), intent(in) :: col
     type(helicity_t), intent(in) :: hel
+    call qn%f%undefine ()
     qn%c = col
     qn%h = hel
   end subroutine quantum_numbers_init0_ch
 
-  subroutine quantum_numbers_init0_fch (qn, flv, col, hel)
-    type(quantum_numbers_t), intent(out) :: qn
+  impure elemental subroutine quantum_numbers_init0_fch (qn, flv, col, hel)
+    class(quantum_numbers_t), intent(inout) :: qn
     type(flavor_t), intent(in) :: flv
     type(color_t), intent(in) :: col
     type(helicity_t), intent(in) :: hel
@@ -258,254 +240,157 @@ contains
     qn%h = hel
   end subroutine quantum_numbers_init0_fch
 
-  subroutine quantum_numbers_init1_f (qn, flv)
-    type(quantum_numbers_t), dimension(:), intent(out) :: qn
-    type(flavor_t), dimension(:), intent(in) :: flv
-    integer :: i
-    do i = 1, size (qn)
-       call quantum_numbers_init0_f (qn(i), flv(i))
-    end do
-  end subroutine quantum_numbers_init1_f
-
-  subroutine quantum_numbers_init1_c (qn, col)
-    type(quantum_numbers_t), dimension(:), intent(out) :: qn
-    type(color_t), dimension(:), intent(in) :: col
-    integer :: i
-    do i = 1, size (qn)
-       call quantum_numbers_init0_c (qn(i), col(i))
-    end do
-  end subroutine quantum_numbers_init1_c
-
-  subroutine quantum_numbers_init1_h (qn, hel)
-    type(quantum_numbers_t), dimension(:), intent(out) :: qn
-    type(helicity_t), dimension(:), intent(in) :: hel
-    integer :: i
-    do i = 1, size (qn)
-       call quantum_numbers_init0_h (qn(i), hel(i))
-    end do
-  end subroutine quantum_numbers_init1_h
-
-  subroutine quantum_numbers_init1_fc (qn, flv, col)
-    type(quantum_numbers_t), dimension(:), intent(out) :: qn
-    type(flavor_t), dimension(:), intent(in) :: flv
-    type(color_t), dimension(:), intent(in) :: col
-    integer :: i
-    do i = 1, size (qn)
-       call quantum_numbers_init0_fc (qn(i), flv(i), col(i))
-    end do
-  end subroutine quantum_numbers_init1_fc
-
-  subroutine quantum_numbers_init1_fh (qn, flv, hel)
-    type(quantum_numbers_t), dimension(:), intent(out) :: qn
-    type(flavor_t), dimension(:), intent(in) :: flv
-    type(helicity_t), dimension(:), intent(in) :: hel
-    integer :: i
-    do i = 1, size (qn)
-       call quantum_numbers_init0_fh (qn(i), flv(i), hel(i))
-    end do
-  end subroutine quantum_numbers_init1_fh
-
-  subroutine quantum_numbers_init1_ch (qn, col, hel)
-    type(quantum_numbers_t), dimension(:), intent(out) :: qn
-    type(color_t), dimension(:), intent(in) :: col
-    type(helicity_t), dimension(:), intent(in) :: hel
-    integer :: i
-    do i = 1, size (qn)
-       call quantum_numbers_init0_ch (qn(i), col(i), hel(i))
-    end do
-  end subroutine quantum_numbers_init1_ch
-
-  subroutine quantum_numbers_init1_fch (qn, flv, col, hel)
-    type(quantum_numbers_t), dimension(:), intent(out) :: qn
-    type(flavor_t), dimension(:), intent(in) :: flv
-    type(color_t), dimension(:), intent(in) :: col
-    type(helicity_t), dimension(:), intent(in) :: hel
-    integer :: i
-    do i = 1, size (qn)
-       call quantum_numbers_init0_fch (qn(i), flv(i), col(i), hel(i))
-    end do
-  end subroutine quantum_numbers_init1_fch
-
-  subroutine quantum_numbers_write_single (qn, unit)
-    type(quantum_numbers_t), intent(in) :: qn
+  subroutine quantum_numbers_write_single (qn, unit, col_verbose)
+    class(quantum_numbers_t), intent(in) :: qn
     integer, intent(in), optional :: unit
+    logical, intent(in), optional :: col_verbose
     integer :: u
+    logical :: col_verb
     u = given_output_unit (unit);  if (u < 0)  return
+    col_verb = .false.;  if (present (col_verbose)) col_verb = col_verbose
     write (u, "(A)", advance="no")  "["
-    if (flavor_is_defined (qn%f)) then
-       call flavor_write (qn%f, u)
-       if (color_is_defined (qn%c) .or. helicity_is_defined (qn%h)) &
+    if (qn%f%is_defined ()) then
+       call qn%f%write (u)
+       if (qn%c%is_nonzero () .or. qn%h%is_defined ()) &
             write (u, "(1x)", advance="no")
     end if
-    if (color_is_defined (qn%c) .or. color_is_ghost (qn%c)) then
-       call color_write (qn%c, u)
-       if (helicity_is_defined (qn%h))  write (u, "(1x)", advance="no")
+    if (col_verb) then
+       if (qn%c%is_defined () .or. qn%c%is_ghost ()) then
+          call color_write (qn%c, u)
+          if (qn%h%is_defined ())  write (u, "(1x)", advance="no")
+       end if
+    else
+       if (qn%c%is_nonzero () .or. qn%c%is_ghost ()) then
+          call color_write (qn%c, u)
+          if (qn%h%is_defined ())  write (u, "(1x)", advance="no")
+       end if
     end if
-    if (helicity_is_defined (qn%h)) then
-       call helicity_write (qn%h, u)
+    if (qn%h%is_defined ()) then
+       call qn%h%write (u)
     end if
     write (u, "(A)", advance="no")  "]"
   end subroutine quantum_numbers_write_single
 
-  subroutine quantum_numbers_write_array (qn, unit)
+  subroutine quantum_numbers_write_array (qn, unit, col_verbose)
     type(quantum_numbers_t), dimension(:), intent(in) :: qn
     integer, intent(in), optional :: unit
+    logical, intent(in), optional :: col_verbose
     integer :: i
     integer :: u
+    logical :: col_verb
     u = given_output_unit (unit);  if (u < 0)  return
+    col_verb = .false.;  if (present (col_verbose)) col_verb = col_verbose
     write (u, "(A)", advance="no")  "["
     do i = 1, size (qn)
        if (i > 1)  write (u, "(A)", advance="no")  " / "
-       if (flavor_is_defined (qn(i)%f)) then
-          call flavor_write (qn(i)%f, u)
-          if (color_is_defined (qn(i)%c) .or. helicity_is_defined (qn(i)%h)) &
+       if (qn(i)%f%is_defined ()) then
+          call qn(i)%f%write (u)
+          if (qn(i)%c%is_nonzero () .or. qn(i)%h%is_defined ()) &
                write (u, "(1x)", advance="no")
        end if
-       if (color_is_defined (qn(i)%c) .or. color_is_ghost (qn(i)%c)) then
-          call color_write (qn(i)%c, u)
-          if (helicity_is_defined (qn(i)%h))  write (u, "(1x)", advance="no")
+       if (col_verb) then
+          if (qn(i)%c%is_defined () .or. qn(i)%c%is_ghost ()) then
+             call color_write (qn(i)%c, u)
+             if (qn(i)%h%is_defined ())  write (u, "(1x)", advance="no")
+          end if
+       else
+          if (qn(i)%c%is_nonzero () .or. qn(i)%c%is_ghost ()) then
+             call color_write (qn(i)%c, u)
+             if (qn(i)%h%is_defined ())  write (u, "(1x)", advance="no")
+          end if
        end if
-       if (helicity_is_defined (qn(i)%h)) then
-          call helicity_write (qn(i)%h, u)
+       if (qn(i)%h%is_defined ()) then
+          call qn(i)%h%write (u)
        end if
     end do
     write (u, "(A)", advance="no")  "]"
   end subroutine quantum_numbers_write_array
 
   subroutine quantum_numbers_write_raw (qn, u)
-    type(quantum_numbers_t), intent(in) :: qn
+    class(quantum_numbers_t), intent(in) :: qn
     integer, intent(in) :: u
-    call flavor_write_raw (qn%f, u)
-    call color_write_raw (qn%c, u)
-    call helicity_write_raw (qn%h, u)
+    call qn%f%write_raw (u)
+    call qn%c%write_raw (u)
+    call qn%h%write_raw (u)
   end subroutine quantum_numbers_write_raw
 
   subroutine quantum_numbers_read_raw (qn, u, iostat)
-    type(quantum_numbers_t), intent(out) :: qn
+    class(quantum_numbers_t), intent(out) :: qn
     integer, intent(in) :: u
     integer, intent(out), optional :: iostat
-    call flavor_read_raw (qn%f, u, iostat=iostat)
-    call color_read_raw (qn%c, u, iostat=iostat)
-    call helicity_read_raw (qn%h, u, iostat=iostat)
+    call qn%f%read_raw (u, iostat=iostat)
+    call qn%c%read_raw (u, iostat=iostat)
+    call qn%h%read_raw (u, iostat=iostat)
   end subroutine quantum_numbers_read_raw
 
-  function quantum_numbers_get_flavor0 (qn) result (flv)
+  impure elemental function quantum_numbers_get_flavor (qn) result (flv)
     type(flavor_t) :: flv
-    type(quantum_numbers_t), intent(in) :: qn
+    class(quantum_numbers_t), intent(in) :: qn
     flv = qn%f
-  end function quantum_numbers_get_flavor0
-
-  function quantum_numbers_get_flavor1 (qn) result (flv)
-    type(quantum_numbers_t), dimension(:), intent(in) :: qn
-    type(flavor_t), dimension(size(qn)) :: flv
-    integer :: i
-    do i = 1, size (qn)
-       flv(i) = qn(i)%f
-    end do
-  end function quantum_numbers_get_flavor1
+  end function quantum_numbers_get_flavor
 
   elemental function quantum_numbers_get_color (qn) result (col)
     type(color_t) :: col
-    type(quantum_numbers_t), intent(in) :: qn
+    class(quantum_numbers_t), intent(in) :: qn
     col = qn%c
   end function quantum_numbers_get_color
 
   elemental function quantum_numbers_get_helicity (qn) result (hel)
     type(helicity_t) :: hel
-    type(quantum_numbers_t), intent(in) :: qn
+    class(quantum_numbers_t), intent(in) :: qn
     hel = qn%h
   end function quantum_numbers_get_helicity
 
-  subroutine quantum_numbers_set_flavor0 (qn, flv)
-    type(quantum_numbers_t), intent(inout) :: qn
-    type(flavor_t), intent(in) :: flv
-    qn%f = flv
-  end subroutine quantum_numbers_set_flavor0
-
-  subroutine quantum_numbers_set_flavor1 (qn, flv)
-    type(quantum_numbers_t), dimension(:), intent(inout) :: qn
-    type(flavor_t), dimension(:), intent(in) :: flv
-    integer :: i
-    do i = 1, size (flv)
-       qn(i)%f = flv(i)
-    end do
-  end subroutine quantum_numbers_set_flavor1
-
-  elemental subroutine quantum_numbers_set_color (qn, col)
-    type(quantum_numbers_t), intent(inout) :: qn
-    type(color_t), intent(in) :: col
-    qn%c = col
-  end subroutine quantum_numbers_set_color
-
-  elemental subroutine quantum_numbers_set_helicity (qn, hel)
-    type(quantum_numbers_t), intent(inout) :: qn
-    type(helicity_t), intent(in) :: hel
-    qn%h = hel
-  end subroutine quantum_numbers_set_helicity
-
   elemental subroutine quantum_numbers_set_color_ghost (qn, ghost)
-    type(quantum_numbers_t), intent(inout) :: qn
+    class(quantum_numbers_t), intent(inout) :: qn
     logical, intent(in) :: ghost
-    call color_set_ghost (qn%c, ghost)
+    call qn%c%set_ghost (ghost)
   end subroutine quantum_numbers_set_color_ghost
 
-  elemental subroutine quantum_numbers_set_helicity_ghost (qn, ghost)
-    type(quantum_numbers_t), intent(inout) :: qn
-    logical, intent(in) :: ghost
-    call helicity_set_ghost (qn%h, ghost)
-  end subroutine quantum_numbers_set_helicity_ghost
-
-  subroutine quantum_numbers_set_model_single (qn, model)
-    type(quantum_numbers_t), intent(inout) :: qn
+  impure elemental subroutine quantum_numbers_set_model (qn, model)
+    class(quantum_numbers_t), intent(inout) :: qn
     class(model_data_t), intent(in), target :: model
-    call flavor_set_model (qn%f, model)
-  end subroutine quantum_numbers_set_model_single
-
-  subroutine quantum_numbers_set_model_array (qn, model)
-    type(quantum_numbers_t), dimension(:), intent(inout) :: qn
-    class(model_data_t), intent(in), target :: model
-    call flavor_set_model (qn%f, model)
-  end subroutine quantum_numbers_set_model_array
+    call qn%f%set_model (model)
+  end subroutine quantum_numbers_set_model
 
+  elemental subroutine quantum_numbers_tag_radiated (qn)
+    class(quantum_numbers_t), intent(inout) :: qn
+    call qn%f%tag_radiated ()
+  end subroutine quantum_numbers_tag_radiated
+  
   elemental function quantum_numbers_get_color_type (qn) result (color_type)
     integer :: color_type
-    type(quantum_numbers_t), intent(in) :: qn
-    color_type = flavor_get_color_type (qn%f)
+    class(quantum_numbers_t), intent(in) :: qn
+    color_type = qn%f%get_color_type ()
   end function quantum_numbers_get_color_type
 
   elemental function quantum_numbers_are_valid (qn) result (valid)
     logical :: valid
-    type(quantum_numbers_t), intent(in) :: qn
-    valid = flavor_is_valid (qn%f)
+    class(quantum_numbers_t), intent(in) :: qn
+    valid = qn%f%is_valid ()
   end function quantum_numbers_are_valid
 
   elemental function quantum_numbers_are_associated (qn) result (flag)
     logical :: flag
-    type(quantum_numbers_t), intent(in) :: qn
-    flag = flavor_is_associated (qn%f)
+    class(quantum_numbers_t), intent(in) :: qn
+    flag = qn%f%is_associated ()
   end function quantum_numbers_are_associated
 
   elemental function quantum_numbers_are_diagonal (qn) result (diagonal)
     logical :: diagonal
-    type(quantum_numbers_t), intent(in) :: qn
-    diagonal = helicity_is_diagonal (qn%h) .and. color_is_diagonal (qn%c)
+    class(quantum_numbers_t), intent(in) :: qn
+    diagonal = qn%h%is_diagonal () .and. qn%c%is_diagonal ()
   end function quantum_numbers_are_diagonal
 
   elemental function quantum_numbers_is_color_ghost (qn) result (ghost)
     logical :: ghost
-    type(quantum_numbers_t), intent(in) :: qn
-    ghost = color_is_ghost (qn%c)
+    class(quantum_numbers_t), intent(in) :: qn
+    ghost = qn%c%is_ghost ()
   end function quantum_numbers_is_color_ghost
 
-  elemental function quantum_numbers_is_helicity_ghost (qn) result (ghost)
-    logical :: ghost
-    type(quantum_numbers_t), intent(in) :: qn
-    ghost = helicity_is_ghost (qn%h)
-  end function quantum_numbers_is_helicity_ghost
-
   elemental function quantum_numbers_match (qn1, qn2) result (match)
     logical :: match
-    type(quantum_numbers_t), intent(in) :: qn1, qn2
+    class(quantum_numbers_t), intent(in) :: qn1, qn2
     match = (qn1%f .match. qn2%f) .and. &
          (qn1%c .match. qn2%c) .and. &
          (qn1%h .match. qn2%h)
@@ -513,20 +398,20 @@ contains
 
   elemental function quantum_numbers_match_f (qn1, qn2) result (match)
     logical :: match
-    type(quantum_numbers_t), intent(in) :: qn1, qn2
+    class(quantum_numbers_t), intent(in) :: qn1, qn2
     match = (qn1%f .match. qn2%f)
   end function quantum_numbers_match_f
 
   elemental function quantum_numbers_match_fh (qn1, qn2) result (match)
     logical :: match
-    type(quantum_numbers_t), intent(in) :: qn1, qn2
+    class(quantum_numbers_t), intent(in) :: qn1, qn2
     match = (qn1%f .match. qn2%f) .and. &
          (qn1%h .match. qn2%h)
   end function quantum_numbers_match_fh
 
   elemental function quantum_numbers_match_hel_diag (qn1, qn2) result (match)
     logical :: match
-    type(quantum_numbers_t), intent(in) :: qn1, qn2
+    class(quantum_numbers_t), intent(in) :: qn1, qn2
     match = (qn1%f .match. qn2%f) .and. &
          (qn1%c .match. qn2%c) .and. &
          (qn1%h .dmatch. qn2%h)
@@ -534,7 +419,7 @@ contains
 
   elemental function quantum_numbers_eq (qn1, qn2) result (eq)
     logical :: eq
-    type(quantum_numbers_t), intent(in) :: qn1, qn2
+    class(quantum_numbers_t), intent(in) :: qn1, qn2
     eq = (qn1%f == qn2%f) .and. &
          (qn1%c == qn2%c) .and. &
          (qn1%h == qn2%h)
@@ -542,7 +427,7 @@ contains
 
   elemental function quantum_numbers_neq (qn1, qn2) result (neq)
     logical :: neq
-    type(quantum_numbers_t), intent(in) :: qn1, qn2
+    class(quantum_numbers_t), intent(in) :: qn1, qn2
     neq = (qn1%f /= qn2%f) .or. &
          (qn1%c /= qn2%c) .or. &
          (qn1%h /= qn2%h)
@@ -559,10 +444,10 @@ contains
        flag = (qn1%f .match. qn2%f)
     end if
     if (mask%c) then
-       flag = flag .and. (color_is_ghost (qn1%c) .eqv. color_is_ghost (qn2%c))
+       flag = flag .and. (qn1%c%is_ghost () .eqv. qn2%c%is_ghost ())
     else
        flag = flag .and. &
-            .not. (color_is_ghost (qn1%c) .or. color_is_ghost (qn2%c)) .and. &
+            .not. (qn1%c%is_ghost () .or. qn2%c%is_ghost ()) .and. &
             (qn1%c == qn2%c)
     end if
   end function quantum_numbers_are_compatible
@@ -574,7 +459,7 @@ contains
     if (mask%c) then
        flag = .true.
     else
-       flag = .not. color_is_ghost (qn%c)
+       flag = .not. qn%c%is_ghost ()
     end if
   end function quantum_numbers_are_physical
 
@@ -583,11 +468,11 @@ contains
     call color_canonicalize (qn%c)
   end subroutine quantum_numbers_canonicalize_color
 
-  subroutine quantum_numbers_set_color_map (map, qn1, qn2)
+  subroutine quantum_numbers_make_color_map (map, qn1, qn2)
     integer, dimension(:,:), intent(out), allocatable :: map
     type(quantum_numbers_t), dimension(:), intent(in) :: qn1, qn2
-    call set_color_map (map, qn1%c, qn2%c)
-  end subroutine quantum_numbers_set_color_map
+    call make_color_map (map, qn1%c, qn2%c)
+  end subroutine quantum_numbers_make_color_map
 
   subroutine quantum_numbers_translate_color0 (qn, map, offset)
     type(quantum_numbers_t), intent(inout) :: qn
@@ -603,28 +488,28 @@ contains
     call color_translate (qn%c, map, offset)
   end subroutine quantum_numbers_translate_color1
 
-  function quantum_numbers_get_max_color_value0 (qn) result (cmax)
+  pure function quantum_numbers_get_max_color_value0 (qn) result (cmax)
     integer :: cmax
     type(quantum_numbers_t), intent(in) :: qn
     cmax = color_get_max_value (qn%c)
   end function quantum_numbers_get_max_color_value0
     
-  function quantum_numbers_get_max_color_value1 (qn) result (cmax)
+  pure function quantum_numbers_get_max_color_value1 (qn) result (cmax)
     integer :: cmax
     type(quantum_numbers_t), dimension(:), intent(in) :: qn
     cmax = color_get_max_value (qn%c)
   end function quantum_numbers_get_max_color_value1
     
-  function quantum_numbers_get_max_color_value2 (qn) result (cmax)
+  pure function quantum_numbers_get_max_color_value2 (qn) result (cmax)
     integer :: cmax
     type(quantum_numbers_t), dimension(:,:), intent(in) :: qn
     cmax = color_get_max_value (qn%c)
   end function quantum_numbers_get_max_color_value2
     
   elemental subroutine quantum_numbers_add_color_offset (qn, offset)
-    type(quantum_numbers_t), intent(inout) :: qn
+    class(quantum_numbers_t), intent(inout) :: qn
     integer, intent(in) :: offset
-    call color_add_offset (qn%c, offset)
+    call qn%c%add_offset (offset)
   end subroutine quantum_numbers_add_color_offset
 
   subroutine quantum_number_array_make_color_contractions (qn_in, qn_out)
@@ -642,8 +527,8 @@ contains
   end subroutine quantum_number_array_make_color_contractions
 
   elemental subroutine quantum_numbers_invert_color (qn)
-    type(quantum_numbers_t), intent(inout) :: qn
-    call color_invert (qn%c)
+    class(quantum_numbers_t), intent(inout) :: qn
+    call qn%c%invert ()
   end subroutine quantum_numbers_invert_color
 
   function merge_quantum_numbers0 (qn1, qn2) result (qn3)
@@ -662,7 +547,7 @@ contains
     qn3%h = qn1%h .merge. qn2%h
   end function merge_quantum_numbers1
 
-  elemental function new_quantum_numbers_mask &
+  elemental function quantum_numbers_mask &
        (mask_f, mask_c, mask_h, mask_cg, mask_hd) result (mask)
     type(quantum_numbers_mask_t) :: mask
     logical, intent(in) :: mask_f, mask_c, mask_h
@@ -670,28 +555,30 @@ contains
     logical, intent(in), optional :: mask_hd
     call quantum_numbers_mask_init &
          (mask, mask_f, mask_c, mask_h, mask_cg, mask_hd)
-  end function new_quantum_numbers_mask
+  end function quantum_numbers_mask
 
   elemental subroutine quantum_numbers_mask_init &
        (mask, mask_f, mask_c, mask_h, mask_cg, mask_hd)
-    type(quantum_numbers_mask_t), intent(out) :: mask
+    class(quantum_numbers_mask_t), intent(inout) :: mask
     logical, intent(in) :: mask_f, mask_c, mask_h
     logical, intent(in), optional :: mask_cg, mask_hd
     mask%f = mask_f
     mask%c = mask_c
     mask%h = mask_h
+    mask%cg = .false.
     if (present (mask_cg)) then
        if (mask%c)  mask%cg = mask_cg
     else
        mask%cg = mask_c
     end if
+    mask%hd = .false.
     if (present (mask_hd)) then
        if (.not. mask%h)  mask%hd = mask_hd
     end if
   end subroutine quantum_numbers_mask_init
 
   subroutine quantum_numbers_mask_write_single (mask, unit)
-    type(quantum_numbers_mask_t), intent(in) :: mask
+    class(quantum_numbers_mask_t), intent(in) :: mask
     integer, intent(in), optional :: unit
     integer :: u
     u = given_output_unit (unit);  if (u < 0)  return
@@ -722,13 +609,13 @@ contains
   end subroutine quantum_numbers_mask_write_array
 
   elemental subroutine quantum_numbers_mask_set_flavor (mask, mask_f)
-    type(quantum_numbers_mask_t), intent(inout) :: mask
+    class(quantum_numbers_mask_t), intent(inout) :: mask
     logical, intent(in) :: mask_f
     mask%f = mask_f
   end subroutine quantum_numbers_mask_set_flavor
 
   elemental subroutine quantum_numbers_mask_set_color (mask, mask_c, mask_cg)
-    type(quantum_numbers_mask_t), intent(inout) :: mask
+    class(quantum_numbers_mask_t), intent(inout) :: mask
     logical, intent(in) :: mask_c
     logical, intent(in), optional :: mask_cg
     mask%c = mask_c
@@ -740,7 +627,7 @@ contains
   end subroutine quantum_numbers_mask_set_color
 
   elemental subroutine quantum_numbers_mask_set_helicity (mask, mask_h, mask_hd)
-    type(quantum_numbers_mask_t), intent(inout) :: mask
+    class(quantum_numbers_mask_t), intent(inout) :: mask
     logical, intent(in) :: mask_h
     logical, intent(in), optional :: mask_hd
     mask%h = mask_h
@@ -751,8 +638,8 @@ contains
 
   elemental subroutine quantum_numbers_mask_assign &
        (mask, mask_in, flavor, color, helicity)
-    type(quantum_numbers_mask_t), intent(inout) :: mask
-    type(quantum_numbers_mask_t), intent(in) :: mask_in
+    class(quantum_numbers_mask_t), intent(inout) :: mask
+    class(quantum_numbers_mask_t), intent(in) :: mask_in
     logical, intent(in), optional :: flavor, color, helicity
     if (present (flavor)) then
        if (flavor) then
@@ -780,18 +667,18 @@ contains
   end function quantum_numbers_mask_any
 
   elemental function quantum_numbers_mask_or (mask1, mask2) result (mask)
-     type(quantum_numbers_mask_t) :: mask
-     type(quantum_numbers_mask_t), intent(in) :: mask1, mask2
-     mask%f = mask1%f .or. mask2%f
-     mask%c = mask1%c .or. mask2%c
-     if (mask%c)  mask%cg = mask1%cg .or. mask2%cg
-     mask%h = mask1%h .or. mask2%h
-     if (.not. mask%h)  mask%hd = mask1%hd .or. mask2%hd
-   end function quantum_numbers_mask_or
+    type(quantum_numbers_mask_t) :: mask
+    class(quantum_numbers_mask_t), intent(in) :: mask1, mask2
+    mask%f = mask1%f .or. mask2%f
+    mask%c = mask1%c .or. mask2%c
+    if (mask%c)  mask%cg = mask1%cg .or. mask2%cg
+    mask%h = mask1%h .or. mask2%h
+    if (.not. mask%h)  mask%hd = mask1%hd .or. mask2%hd
+  end function quantum_numbers_mask_or
 
   elemental function quantum_numbers_mask_eqv (mask1, mask2) result (eqv)
     logical :: eqv
-    type(quantum_numbers_mask_t), intent(in) :: mask1, mask2
+    class(quantum_numbers_mask_t), intent(in) :: mask1, mask2
     eqv = (mask1%f .eqv. mask2%f) .and. &
          (mask1%c .eqv. mask2%c) .and. &
          (mask1%cg .eqv. mask2%cg) .and. &
@@ -801,7 +688,7 @@ contains
 
   elemental function quantum_numbers_mask_neqv (mask1, mask2) result (neqv)
     logical :: neqv
-    type(quantum_numbers_mask_t), intent(in) :: mask1, mask2
+    class(quantum_numbers_mask_t), intent(in) :: mask1, mask2
     neqv = (mask1%f .neqv. mask2%f) .or. &
          (mask1%c .neqv. mask2%c) .or. &
          (mask1%cg .neqv. mask2%cg) .or. &
@@ -810,24 +697,26 @@ contains
   end function quantum_numbers_mask_neqv
 
   elemental subroutine quantum_numbers_undefine (qn, mask)
-    type(quantum_numbers_t), intent(inout) :: qn
+    class(quantum_numbers_t), intent(inout) :: qn
     type(quantum_numbers_mask_t), intent(in) :: mask
-    if (mask%f)  call flavor_undefine (qn%f)
-    if (mask%c)  call color_undefine (qn%c, undefine_ghost=mask%cg)
+    if (mask%f)  call qn%f%undefine ()
+    if (mask%c)  call qn%c%undefine (undefine_ghost=mask%cg)
     if (mask%h) then
-       call helicity_undefine (qn%h)
+       call qn%h%undefine ()
     else if (mask%hd) then
-       if (.not. helicity_is_diagonal (qn%h)) then
-          call helicity_diagonalize (qn%h)
+       if (.not. qn%h%is_diagonal ()) then
+          call qn%h%diagonalize ()
        end if
     end if
   end subroutine quantum_numbers_undefine
 
   function quantum_numbers_undefined0 (qn, mask) result (qn_new)
-    type(quantum_numbers_t), intent(in) :: qn
+    class(quantum_numbers_t), intent(in) :: qn
     type(quantum_numbers_mask_t), intent(in) :: mask
     type(quantum_numbers_t) :: qn_new
-    qn_new = qn
+    select type (qn)
+    type is (quantum_numbers_t);  qn_new = qn
+    end select
     call quantum_numbers_undefine (qn_new, mask)
   end function quantum_numbers_undefined0
 
@@ -850,26 +739,26 @@ contains
   elemental function quantum_numbers_are_redundant (qn, mask) &
        result (redundant)
     logical :: redundant
-    type(quantum_numbers_t), intent(in) :: qn
+    class(quantum_numbers_t), intent(in) :: qn
     type(quantum_numbers_mask_t), intent(in) :: mask
     redundant = .false.
     if (mask%f) then
-       redundant = flavor_is_defined (qn%f)
+       redundant = qn%f%is_defined ()
     end if
     if (mask%c) then
-       redundant = color_is_defined (qn%c)
+       redundant = qn%c%is_defined ()
     end if
     if (mask%h) then
-       redundant = helicity_is_defined (qn%h)
+       redundant = qn%h%is_defined ()
     else if (mask%hd) then
-       redundant = .not. helicity_is_diagonal (qn%h)
+       redundant = .not. qn%h%is_diagonal ()
     end if
   end function quantum_numbers_are_redundant
 
   elemental function quantum_numbers_mask_diagonal_helicity (mask) &
        result (flag)
     logical :: flag
-    type(quantum_numbers_mask_t), intent(in) :: mask
+    class(quantum_numbers_mask_t), intent(in) :: mask
     flag = mask%h .or. mask%hd
   end function quantum_numbers_mask_diagonal_helicity
 
