@@ -1,6 +1,6 @@
-! WHIZARD 2.2.3 Nov 30 2014
+! WHIZARD 2.2.5 Feb 27 2015
 ! 
-! Copyright (C) 1999-2014 by 
+! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
@@ -9,7 +9,8 @@
 !     Fabian Bach <fabian.bach@desy.de>
 !     Christian Speckner <cnspeckn@googlemail.com> 
 !     Christian Weiss <christian.weiss@desy.de>
-!     and Felix Braam, Sebastian Schmidt, Daniel Wiesler 
+!     and Hans-Werner Boschmann, Felix Braam, 
+!     Sebastian Schmidt, Daniel Wiesler 
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -79,6 +80,7 @@ module parton_states
      procedure :: get_trace_int_ptr => parton_state_get_trace_int_ptr
      procedure :: get_matrix_int_ptr => parton_state_get_matrix_int_ptr
      procedure :: get_flows_int_ptr => parton_state_get_flows_int_ptr
+     procedure :: get_n_out => parton_state_get_n_out
   end type parton_state_t
 
   type, extends (parton_state_t) :: isolated_state_t
@@ -96,6 +98,7 @@ module parton_states
 
   public :: connected_state_t
   type, extends (parton_state_t) :: connected_state_t
+     type(state_flv_content_t) :: state_flv
      logical :: has_flows_sf = .false.
      type(evaluator_t) :: flows_sf
      logical :: has_expr = .false.
@@ -104,6 +107,8 @@ module parton_states
      procedure :: setup_connected_trace => connected_state_setup_connected_trace
      procedure :: setup_connected_matrix => connected_state_setup_connected_matrix
      procedure :: setup_connected_flows => connected_state_setup_connected_flows
+     procedure :: setup_state_flv => connected_state_setup_state_flv
+     procedure :: get_state_flv => connected_state_get_state_flv
      procedure :: setup_subevt => connected_state_setup_subevt
      procedure :: setup_var_list => connected_state_setup_var_list
      procedure :: setup_cuts => connected_state_setup_cuts
@@ -230,17 +235,21 @@ contains
     associate (data => core%data)
       allocate (qn_mask (data%n_in + data%n_out))
       qn_mask(:data%n_in) = &
-              new_quantum_numbers_mask (.false., .true., .false.) &
+              quantum_numbers_mask (.false., .true., .false.) &
               .or. qn_mask_in
       qn_mask(data%n_in+1:) = &
-           new_quantum_numbers_mask (.true., .true., .true.)
+           quantum_numbers_mask (.true., .true., .true.)
     if (core%use_color_factors) then
        call evaluator_init_square (state%trace, &
             state%int_eff, qn_mask, &
-            data%cf_index, data%color_factors, col, nc=core%nc)
+            col_flow_index = data%cf_index, &
+            col_factor = data%color_factors, &
+            col_index_hi = col, &
+            nc = core%nc)
     else
        call evaluator_init_square (state%trace, &
-            state%int_eff, qn_mask, nc=core%nc)
+            state%int_eff, qn_mask, &
+            nc = core%nc)
     end if
     end associate
     state%has_trace = .true.
@@ -261,28 +270,32 @@ contains
       allocate (qn_mask (data%n_in + data%n_out))
       allocate (flv (data%n_flv))
       do i = 1, data%n_in + data%n_out      
-         call flavor_init (flv, data%flv_state(i,:), model)
+         call flv%init (data%flv_state(i,:), model)
          if ((data%n_in == 1 .or. i > data%n_in) &
-              .and. any (.not. flavor_is_stable (flv))) then
-            helmask = all (flavor_decays_isotropically (flv))
-            helmask_hd = all (flavor_decays_diagonal (flv))
-            qn_mask(i) = new_quantum_numbers_mask (.false., .true., helmask, &
+              .and. any (.not. flv%is_stable ())) then
+            helmask = all (flv%decays_isotropically ())
+            helmask_hd = all (flv%decays_diagonal ())
+            qn_mask(i) = quantum_numbers_mask (.false., .true., helmask, &
                  mask_hd = helmask_hd)
          else if (i > data%n_in) then
-            helmask = all (.not. flavor_is_polarized (flv))
-            qn_mask(i) = new_quantum_numbers_mask (.false., .true., helmask)
+            helmask = all (.not. flv%is_polarized ())
+            qn_mask(i) = quantum_numbers_mask (.false., .true., helmask)
          else
-            qn_mask(i) = new_quantum_numbers_mask (.false., .true., .false.) &
+            qn_mask(i) = quantum_numbers_mask (.false., .true., .false.) &
               .or. qn_mask_in(i)
          end if
       end do
     if (core%use_color_factors) then
        call evaluator_init_square (state%matrix, &
             state%int_eff, qn_mask, &
-            data%cf_index, data%color_factors, col, nc=core%nc)
+            col_flow_index = data%cf_index, &
+            col_factor = data%color_factors, &
+            col_index_hi = col, &
+            nc = core%nc)
     else
        call evaluator_init_square (state%matrix, state%int_eff, &
-            qn_mask, nc=core%nc)
+            qn_mask, &
+            nc = core%nc)
     end if
     end associate
     state%has_matrix = .true.
@@ -301,18 +314,18 @@ contains
       allocate (qn_mask (data%n_in + data%n_out))
       allocate (flv (data%n_flv))
       do i = 1, data%n_in + data%n_out
-         call flavor_init (flv, data%flv_state(i,:), model)
+         call flv%init (data%flv_state(i,:), model)
          if ((data%n_in == 1 .or. i > data%n_in) &
-              .and. any (.not. flavor_is_stable (flv))) then
-            helmask = all (flavor_decays_isotropically (flv))
-            helmask_hd = all (flavor_decays_diagonal (flv))
-            qn_mask(i) = new_quantum_numbers_mask (.false., .false., helmask, &
+              .and. any (.not. flv%is_stable ())) then
+            helmask = all (flv%decays_isotropically ())
+            helmask_hd = all (flv%decays_diagonal ())
+            qn_mask(i) = quantum_numbers_mask (.false., .false., helmask, &
                  mask_hd = helmask_hd)
          else if (i > data%n_in) then
-            helmask = all (.not. flavor_is_polarized (flv))
-            qn_mask(i) = new_quantum_numbers_mask (.false., .false., helmask)
+            helmask = all (.not. flv%is_polarized ())
+            qn_mask(i) = quantum_numbers_mask (.false., .false., helmask)
          else
-            qn_mask(i) = new_quantum_numbers_mask (.false., .false., .false.) &
+            qn_mask(i) = quantum_numbers_mask (.false., .false., .false.) &
               .or. qn_mask_in(i)
          end if
       end do
@@ -330,27 +343,30 @@ contains
     logical, intent(in), optional :: resonant
     type(quantum_numbers_mask_t) :: mask
     type(interaction_t), pointer :: src_int
-    mask = new_quantum_numbers_mask (.true., .true., .true.)
+    mask = quantum_numbers_mask (.true., .true., .true.)
     if (present (int)) then
        src_int => int
     else
        src_int => isolated%sf_chain_eff%get_out_int_ptr ()
     end if
     call evaluator_init_product &
-         (state%trace, src_int, isolated%trace, mask, mask, &
+         (state%trace, src_int, isolated%trace, &
+          qn_mask_conn = mask, &
+          qn_mask_rest = mask, &
           connections_are_resonant = resonant)
     state%has_trace = .true.
   end subroutine connected_state_setup_connected_trace
     
   subroutine connected_state_setup_connected_matrix &
-       (state, isolated, int, resonant)
+       (state, isolated, int, resonant, qn_filter_conn)
     class(connected_state_t), intent(inout), target :: state
     type(isolated_state_t), intent(in), target :: isolated
     type(interaction_t), intent(in), optional, target :: int
     logical, intent(in), optional :: resonant
+    type(quantum_numbers_t), intent(in), optional :: qn_filter_conn
     type(quantum_numbers_mask_t) :: mask
     type(interaction_t), pointer :: src_int
-    mask = new_quantum_numbers_mask (.false., .true., .true.)
+    mask = quantum_numbers_mask (.false., .true., .true.)
     if (present (int)) then
        src_int => int
     else
@@ -358,19 +374,21 @@ contains
     end if
     call evaluator_init_product &
          (state%matrix, src_int, isolated%matrix, mask, &
+          qn_filter_conn = qn_filter_conn, &
           connections_are_resonant = resonant)
     state%has_matrix = .true.
   end subroutine connected_state_setup_connected_matrix
   
   subroutine connected_state_setup_connected_flows &
-       (state, isolated, int, resonant)
+       (state, isolated, int, resonant, qn_filter_conn)
     class(connected_state_t), intent(inout), target :: state
     type(isolated_state_t), intent(in), target :: isolated
     type(interaction_t), intent(in), optional, target :: int
     logical, intent(in), optional :: resonant
+    type(quantum_numbers_t), intent(in), optional :: qn_filter_conn
     type(quantum_numbers_mask_t) :: mask
     type(interaction_t), pointer :: src_int
-    mask = new_quantum_numbers_mask (.false., .false., .true.)
+    mask = quantum_numbers_mask (.false., .false., .true.)
     if (present (int)) then
        src_int => int
     else
@@ -381,10 +399,25 @@ contains
     end if
     call evaluator_init_product &
          (state%flows, src_int, isolated%flows, mask, &
+          qn_filter_conn = qn_filter_conn, &
           connections_are_resonant = resonant)
     state%has_flows = .true.
   end subroutine connected_state_setup_connected_flows
   
+  subroutine connected_state_setup_state_flv (state, n_out_hard)
+    class(connected_state_t), intent(inout), target :: state
+    integer, intent(in) :: n_out_hard
+    type(interaction_t), pointer :: int
+    int => evaluator_get_int_ptr (state%matrix)
+    call interaction_get_flv_content (int, state%state_flv, n_out_hard)
+  end subroutine connected_state_setup_state_flv
+  
+  function connected_state_get_state_flv (state) result (state_flv)
+    class(connected_state_t), intent(in) :: state
+    type(state_flv_content_t) :: state_flv
+    state_flv = state%state_flv
+  end function connected_state_get_state_flv
+  
   subroutine connected_state_setup_subevt (state, sf_chain, f_beam, f_in, f_out)
     class(connected_state_t), intent(inout), target :: state
     type(sf_chain_instance_t), intent(in), target :: sf_chain
@@ -484,12 +517,14 @@ contains
   end subroutine parton_state_send_kinematics
 
   subroutine connected_state_evaluate_expressions (state, passed, &
-       scale, fac_scale, ren_scale, weight)
+       scale, fac_scale, ren_scale, weight, scale_forced)
     class(connected_state_t), intent(inout) :: state
     logical, intent(out) :: passed
     real(default), intent(out) :: scale, fac_scale, ren_scale, weight
+    real(default), intent(in), allocatable, optional :: scale_forced
     if (state%has_expr) then
-       call state%expr%evaluate (passed, scale, fac_scale, ren_scale, weight)
+       call state%expr%evaluate (passed, scale, fac_scale, ren_scale, weight, &
+            scale_forced)
     end if
   end subroutine connected_state_evaluate_expressions
     
@@ -576,5 +611,13 @@ contains
     call state%expr%get_in_index (i_in)
   end subroutine connected_state_get_in_index
   
+  function parton_state_get_n_out (state) result (n)
+    class(parton_state_t), intent(in), target :: state
+    integer :: n
+    type(interaction_t), pointer :: int
+    int => evaluator_get_int_ptr (state%trace)
+    n = interaction_get_n_out (int)
+  end function parton_state_get_n_out
+  
 
 end module parton_states
